package app.game;

import GL.system.screen.Screen;
import app.MusicManager;
import app.ataka.AtakaSys;
import app.game.background.Background;
import app.game.info.Info;
import app.world.layer.Ground;
import app.player.Player;
import app.item.ItemsSys;
import app.item.SysObjectMap;
import app.player.Cursor;
import app.player.Listener;
import app.block.BlockSys;
import app.file.ScreenFile;
import app.inventory.InvScreen;
import app.mobs.GenerateMobs;
import app.mobs.Mob;
import app.mobs.MobRender;
import app.mobs.MobSys;
import app.shop.ShopScreen;
import java.awt.Graphics2D;
import ru.wladimir.map.Map;

/**
 * @author Death
 */
public class Game extends Screen{

    public Player player;
    public InvScreen invScreen;
    public Cursor cursor;
    public Info info;
    public Background background;
    public ScreenFile screenFile;
    public ShopScreen shopScreen;
    
    public Map map;
    public Ground ground;
    
    boolean infoFlagVisible;
    
    public Game() {
        super();
        player = new Player();
        cursor = new Cursor();
        info = new Info();
        background = new Background();
        invScreen = new InvScreen();
        screenFile = new ScreenFile();
        shopScreen = new ShopScreen();
        
        ground = new Ground(Ground.NORMAL);
        
        map = new Map();
    }
    
    public void init(){
        MusicManager.play("game");
        SysObjectMap.init();
        MobSys.init();
        MobRender.init();
        Listener.init();
        ItemsSys.init();
        BlockSys.init();
        AtakaSys.init();
        GenerateMobs.init();
        
        infoFlagVisible = false;
        
        map.add(player);
        ground.layer.objectMapList.add(cursor);
        map.layer.add(ground.layer);
        
        this.addRender(background);
        
        ground.init();
        cursor.init();
        player.init();
        shopScreen.init();
        System.out.println("initGameEnd");
    }

    @Override
    public void paintGUI(Graphics2D g){
        if(invScreen.getVisible()){
            invScreen.paintGUI(g);
        }else if(shopScreen.vis){
            shopScreen.paint(g);
        }else{
            map.paint(g);
            AtakaSys.paint(g);
            info.paint(g);
            GenerateMobs.paint(g);
            if(screenFile.visible){
                screenFile.paint(g);
            }
        }
    }
    @Override
    public void updateGUI() {
        if(invScreen.getVisible()){
            invScreen.updateGUI();
        }else if(shopScreen.vis){
            shopScreen.update();
        }else{
            map.update();
            AtakaSys.update();
            info.update();
            GenerateMobs.update();
            if(screenFile.visible){
                screenFile.update();
            }
        }
    }
    
}
