package app.mobs;

import GL.system.SM;
import app.Manager;
import app.ataka.AtakaObj;
import app.ataka.AtakaSys;
import app.block.Block;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import ru.wladimir.map.Layer;
import static ru.wladimir.map.Setting.SizeTileA;
import ru.wladimir.map.Sprite;

/**
 * @author Death
 */
public class Mob extends Sprite{
    
    public static final Mob mob0 = new Mob().init(1, "MOB", 1, SizeTileA, SM.rnd(3,5), 10);
    public static final Mob mob_1 = (Mob)new Mob_1(2);
    
    public int health, healthMax;
    public int maxBounce;
    public int speed;
    public int ataka;
    public int moneyThis;
    public String name;
    public int id;
    
    public boolean bounce;
    public int iHelp;
    long timeDelay, timeDelayAct, timeDelayMotion, timeDelayBounce, timeDelayAtaka;
    
    public Mob(){
        super();
        healthMax = 10;
        health = healthMax;
        x = 0;
        y = 0;
        w = SizeTileA;
        h = SizeTileA;
        maxBounce = 6;
        speed = SizeTileA;
        iHelp = 0;
        name = "MOB";
        
        timeDelay = 0;
        timeDelayAct = 0;
        timeDelayAtaka = 0;
        timeDelayBounce = 0;
        timeDelayMotion = 0;
        moneyThis = SM.rnd(5, 10);
    }
    
    public Mob init(int id, String name, int ataka, int speed, int mon, int maxHealth){
        this.id = id;
        this.name = name;
        this.ataka = ataka;
        this.speed = speed;
        this.moneyThis = mon;
        this.maxBounce = 0;
        this.health = this.healthMax;
        MobSys.add(this);
        x = SM.rnd(5, Manager.game.ground.layer.w - 5) * SizeTileA;
        y = ySoil();
        return this;
    }
    
    public int ySoil(){
        int y1 = 0;
        while(!isSolid(0, y1 + h)){          
            y1 += 1;
        }
        return y1;
    }

    @Override
    public void paint(Graphics2D g){
        int x1 = this.getXScreen(), y1 = this.getYScreen();
        g.setColor(Color.red);
        g.setStroke(new BasicStroke(3));
        g.drawRect(x1, y1, w, h);
        g.setStroke(new BasicStroke(1));
        paintHealt(g);
    }

    @Override
    public void update(){
        if(!(System.currentTimeMillis() - timeDelay > 1000/20))
            return;
        timeDelay = System.currentTimeMillis();
        
        physics();
        motion();
    }
    
    public Mob cop(){
        return new Mob().init(id, name, ataka, speed, x, health);
    }
    protected void paintHealt(Graphics2D g){
        int w1 = 60;
        int h1 = 15;
        int x1 = (this.getXScreen() + SizeTileA / 2) - w1 / 2;
        int y1 = this.getYScreen() - h1 - 5;
        g.setColor(new Color(255, 0 ,0, 150));
        g.fillRect(x1, y1, w1, h1);
        
        double w2 = 0;
        
        double proZ = ((double)health / ((double)healthMax / 100));
        w2 = ((double)w1 / 100 * proZ);
        
        g.setColor(new Color(0, 255, 0, 150));
        g.fillRect(x1, y1, (int)w2, h1); 
        
        g.setColor(new Color(0, 0, 255, 150));
        g.drawRect(x1 - 1, y1 - 1, w1 + 1, h1 + 1);
        
        Font f = g.getFont();
        g.setFont(new Font("calibri", 0, h1-4));
        String str = String.valueOf(proZ + "%");
        g.drawString(str, x1 + w1/2 - SM.getWidthStr(g, str) / 2, y1 + h1 - 4);
        g.setFont(f);
    }
    
    public void motion(){
        if(!(System.currentTimeMillis() - timeDelayMotion > 1000))
            return;
        timeDelayMotion = System.currentTimeMillis();
        int i = SM.rnd(0, 5);
        switch(i){
            case 1: moveUp(); break;
            case 2: moveDown(); break;
            case 3: moveRight(); break;
            case 4: moveLeft(); break;
        }
    }
    private void moveUp(){
        if(!isSolid(1, -1) && !goingBeyond(w, -1))
            if(!bounce){
                if(System.currentTimeMillis() - timeDelayBounce > 200){
                    timeDelayBounce = System.currentTimeMillis();
                    bounce = true;
                    iHelp = 0;
                }
            }
    }
    private void moveDown(){
        if(      !isSolid(0, h) && !isSolid(w, h) &&
                 !goingBeyond(0, h) && !goingBeyond(w, h))
        {
            move(0, speed);
        }
    }
    private void moveRight(){
        if(     !isSolid(SizeTileA, 0) && !isSolid(SizeTileA, h - SizeTileA) && !isSolid(SizeTileA, h - SizeTileA * 2) && 
                !goingBeyond(w,0) && !goingBeyond(SizeTileA ,h - SizeTileA))
        {
            move(speed, 0);
        }
    }
    private void moveLeft(){
        if(     !isSolid(-speed, 0) && !isSolid(-speed, h - SizeTileA) && !isSolid(-speed, h - SizeTileA * 2) &&
                !goingBeyond(0, 0) && !goingBeyond(0, h - SizeTileA))
        {
            move(-speed, 0);
        }
    }
    
    public void physics(){
        if(bounce && iHelp < maxBounce && !isSolid(0, -SizeTileA/2)){
            iHelp ++;
            move(0, -SizeTileA/2);
            switch(SM.rnd(0, 3)){
                case 1: moveLeft(); break;
                case 2: moveRight(); break;
            }
            timeDelayBounce = System.currentTimeMillis();
        }else{
            if(!isSolid(0, h) && !isSolid(w - 1, h)){
                move(0, SizeTileA/2);
                switch(SM.rnd(0, 3)){
                    case 1: moveLeft(); break;
                    case 2: moveRight(); break;
                }
            }else{
                bounce = false;
            }
        }
    }
    
    public void actMob(){
        if(System.currentTimeMillis() - timeDelayAct > 100){
            timeDelayAct = System.currentTimeMillis();
            act();
        }
    }
    
    public void atakaMob(int a){
        if(System.currentTimeMillis() - timeDelayAtaka > 100){
            timeDelayAtaka = System.currentTimeMillis();
            ataka(a);
        }
    }
    
    public void act(){
        
    }
    
    public void ataka(int ataka){
        healthMinus(ataka);
    }
    
    public void healthMinus(int ataka){
        AtakaObj atakaObj = new AtakaObj();
        AtakaSys.add(atakaObj);
        atakaObj.start(this, ataka);
        this.health -= ataka;
        if(health <= 0){
            MobRender.remove(this);
            Manager.game.player.money += moneyThis;
        }
    }
    
    private boolean isSolid(int dx, int dy){
        Layer l = Manager.game.ground.layer;
        Block b = null;
        if(!goingBeyond(dx, dy)) b = (Block) l.getTile((x + dx) / SizeTileA, (y + dy) / SizeTileA);
        if(b != null && b.getType() == Block.SOLID){
            return true;
        }
        return false;
    }
    private boolean goingBeyond(int dx, int dy){
        if(x + dx <= 0){
            return true;
        }
        if(y + dy <= 0){
            return true;
        }
        if(x + dx >= Manager.game.ground.layer.w * SizeTileA){
            return true;
        }
        if(y + dy >= Manager.game.ground.layer.h * SizeTileA){
            return true;
        }
        return false;
    }
    
}
