package app.player;

import GL.system.Display;
import GL.system.SM;
import app.Manager;
import app.ataka.AtakaObj;
import app.ataka.AtakaSys;
import app.item.ItemsSys;
import app.item.StackItems;
import app.item.SysObjectMap;
import app.block.Block;
import app.block.BlockSys;
import app.inventory.Inventory;
import app.inventory.InventoryBar;
import app.player.body.Body;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import ru.wladimir.map.Layer;
import ru.wladimir.map.ObjectMap;
import static ru.wladimir.map.Setting.SizeTileA;
import ru.wladimir.map.Sprite;

/**
 * @author Death
 */
public class Player extends Sprite{

    public static final int 
            SURVIVAL = 1,
            CREATIVE = 2;
    
    public Inventory inventory;
    public InventoryBar inventoryBar;
    
    public int maxBounce;
    public int gameMode;
    public int delayBreaking;
    public int speed;
    public int ataka, rAtaka;
    public int collectionItemR;
    public int health, healthMax;
    public int rHoldoutBlock;
    
    Color color;
    long timeDelay, timeDelayBounce;
    int sleepBounce;
    public boolean wK, sK, dK, aK;
    public boolean bounce;
    public int iHelp;
    public String name;
    public int money;
    
    public Body body;
    
    public Player() {
        super();
        color = new Color(255, 0, 0);
        inventory = new Inventory();
        inventoryBar = new InventoryBar();
        body = new Body(this);
    }
    public void init(){
        name = "Player";
        w = Body.W;
        h = Body.H;
        x = (Manager.game.ground.layer.w / 2) * SizeTileA;
        y = ySoil();
        Manager.game.map.x = -(Display.getWidth() + x);
        bounce = false;
        iHelp = 0;
        timeDelay = 0;
        timeDelayBounce = 0;
        sleepBounce = 100;
        Manager.game.map.x = Display.getWidth() - x;
        Manager.game.map.y = Display.getHeight() - y;
        inventory.init();
        inventoryBar.init();
        maxBounce = 0;
        delayBreaking = 0;
        money = 0;
        ataka = 1;
        rAtaka = SizeTileA * 3;
        healthMax = 50;
        health = healthMax;
        rHoldoutBlock = SizeTileA * 6;
        setGameMode(SURVIVAL);
    }
    public int ySoil(){
        int y1 = 0;
        while(!isSolid(0, y1 + h)){          
            y1 += 1;
        }
        return y1;
    }
    
    @Override
    public void paint(Graphics2D g) {
        body.paint(g);
        inventoryBar.paint(g);
        paintHealt(g);
        paintMoney(g);
    }
    @Override
    public void update() {
        super.update();
        if(!(System.currentTimeMillis() - timeDelay > 1000/20))
            return;
        timeDelay = System.currentTimeMillis();
        
        motion();
        physics();
        goingBeyond();
        collectingItems();
        body.update();
    }
    
    private void paintHealt(Graphics2D g){
        int w1 = inventoryBar.image.getWidth() / 2 - 10;
        int h1 = 20;
        int x1 = inventoryBar.x + 5;
        int y1 = inventoryBar.y - h1 - 5;
        g.setColor(new Color(255, 0 ,0));
        g.fillRect(x1, y1, w1, h1);
        
        double w2 = 0;
        
        double proZ = ((double)health / ((double)healthMax / 100));
        w2 = ((double)w1 / 100 * proZ);
        
        g.setColor(new Color(0, 255, 0));
        g.fillRect(x1, y1, (int)w2, h1); 
        
        g.setColor(new Color(0, 0, 255));
        g.drawRect(x1 - 1, y1 - 1, w1 + 1, h1 + 1);
        
        Font f = g.getFont();
        g.setFont(new Font("calibri", 0, h1-4));
        String str = String.valueOf(proZ + "%");
        g.drawString(str, x1 + w1/2 - SM.getWidthStr(g, str) / 2, y1 + h1 - 4);
        g.setFont(f);
    }
    
    private void paintMoney(Graphics2D g){
        int w1 = inventoryBar.image.getWidth() / 2 - 10;
        int h1 = 20;
        int x1 = inventoryBar.x + 10 + (inventoryBar.image.getWidth() / 2 - 10);
        int y1 = inventoryBar.y - h1 - 5;
        
        g.setColor(new Color(150, 150, 150));
        g.fillRect(x1, y1, w1, h1);
        g.setColor(new Color(70, 70, 70));
        g.setStroke(new BasicStroke(3));
        g.drawRect(x1, y1, w1, h1);
        g.setStroke(new BasicStroke(1));
        Font f = g.getFont();
        g.setFont(new Font("calibri", 0, 17));
        g.setColor(new Color(10, 10, 10));
        String st = String.valueOf(Manager.game.player.money + " монет");
        g.drawString(st, x1 + w1/2 - SM.getWidthStr(g, st)/2, y1 + h1 - (int)(SM.getHeightStr(g, st) * 0.333));
        g.setFont(f);
    }
    
    public void motion(){
        moveUp();
        moveDown();
        moveRight();
        moveLeft();
    }
    private void moveUp(){
        if((Listener.keyGame.up || wK) && !isSolid(1, -1) && !goingBeyond(w, -1))
            if(!bounce){
                if(System.currentTimeMillis() - timeDelayBounce > sleepBounce){
                    timeDelayBounce = System.currentTimeMillis();
                    bounce = true;
                    iHelp = 0;
                }
            }
    }
    private void moveDown(){
        if((Listener.keyGame.down || sK) && !isSolid(0, h) && !isSolid(w, h) && 
                !goingBeyond(0, h) && !goingBeyond(w, h))
        {
            move(0, speed);
            body.movePlayer();
        }
    }
    private void moveRight(){
        if((Listener.keyGame.right || dK) && 
                !isSolid(SizeTileA, 0) && !isSolid(SizeTileA, h - SizeTileA) && !isSolid(SizeTileA, h - SizeTileA * 2) && 
                !goingBeyond(w,0) && !goingBeyond(SizeTileA ,h - SizeTileA))
        {
            move(speed, 0);
            body.movePlayer();
        }
    }
    private void moveLeft(){
        if((Listener.keyGame.left || aK) && 
                !isSolid(-speed, 0) && !isSolid(-speed, h - SizeTileA) && !isSolid(-speed, h - SizeTileA * 2) &&
                !goingBeyond(0, 0) && !goingBeyond(0, h - SizeTileA))
        {
            move(-speed, 0);
            body.movePlayer();
        }
    }
    
    public boolean holdoutBlock(Cursor cur){
        if(     cur.xId * SizeTileA > this.x - rHoldoutBlock && 
                cur.yId * SizeTileA > this.y - rHoldoutBlock &&
                cur.xId * SizeTileA < this.x + SizeTileA + rHoldoutBlock && 
                cur.yId * SizeTileA < this.y + SizeTileA + rHoldoutBlock){
            return true;
        }
        return false;
    }
    public void healthMinus(int ataka){
        AtakaObj atakaObj = new AtakaObj();
        AtakaSys.add(atakaObj);
        atakaObj.start(this, ataka);
        this.health -= ataka;
        if(health <= 0){
            System.out.println("Смерть!!!");
            System.exit(0);
        }
    }
    public void goingBeyond(){
        if(this.getXScreen() + w >= Display.getWidth() - Display.getWidth() / 2 + w){
            Manager.game.map.x -= speed;
        }
        if(this.getXScreen() <= Display.getWidth() / 2 - w){
            Manager.game.map.x += speed;
        }
        
        if(this.getYScreen() + h >= (Display.getHeight() - Display.getHeight() / 2 - h)){
            Manager.game.map.y -= speed;
        }
        if(this.getYScreen() <= Display.getHeight()/ 2 - h){
            Manager.game.map.y += speed;
        }
        
        if(x <= 0){
            x = 0;
        }
        if(y <= 0){
            y = 0;
        }
        if(x + w > Manager.game.ground.layer.w * SizeTileA){
            x = Manager.game.ground.layer.w * SizeTileA - w;
            bounce = false;
        }
        if(y + h > Manager.game.ground.layer.h * SizeTileA){
            y = Manager.game.ground.layer.h * SizeTileA - h;
            bounce = false;
        }
        
        Manager.game.map.goingBeyond();
    }
    public void physics(){
        if(bounce && iHelp < maxBounce && !isSolid(0, -SizeTileA/2)){
            iHelp ++;
            move(0, -SizeTileA/2);
            body.movePlayer();
            timeDelayBounce = System.currentTimeMillis();
        }else{
            if(!isSolid(0, h) && !isSolid(w - 1, h)){
                move(0, SizeTileA/2);
                body.movePlayer();
            }else{
                bounce = false;
            }
        }
    }
    public void collectingItems(){
        collectionItemR = SizeTileA * 2;
        this.x -= collectionItemR;
        this.y -= collectionItemR;
        this.w += collectionItemR * 2;
        this.h += collectionItemR * 2;
        for(int i = 0; i < SysObjectMap.itemsOb.size(); i++){
            if(SysObjectMap.itemsOb.get(i).getTouching(this) != ObjectMap.NOPE){
                StackItems s = SysObjectMap.itemsOb.get(i).stack;
                boolean flagS = false;
                int num = 0;
                for(int u = 0; u < s.items.size(); u++){
                    if(inventoryBar.addItem(s.items.get(u))){
                        flagS = true;
                    }else{
                        if(inventory.addItem(s.items.get(u))){
                            flagS = true;
                        }else{
                            flagS = false;
                            num = s.items.size() - u;
                        }
                    }
                }
                if(flagS){
                    SysObjectMap.remove(SysObjectMap.itemsOb.get(i));
                }else{
                    StackItems s1 = new StackItems(ItemsSys.getItemById(s.idItems));
                    for(int r = 0; r < num; r++){
                        s1.add(ItemsSys.getNewItem(ItemsSys.getItemById(s.idItems)));
                    }
                }
            }
        }
        this.x += collectionItemR;
        this.y += collectionItemR;
        this.w -= collectionItemR * 2;
        this.h -= collectionItemR * 2;
    }
    public void setGameMode(int mode){
        if(mode == CREATIVE){
            maxBounce = 10;
            gameMode = mode;
            delayBreaking = 1;
            speed = SizeTileA;
            collectionItemR = 40;
        }
        if(mode == SURVIVAL){
            maxBounce = 5;
            gameMode = mode;
            delayBreaking = 100;
            speed = SizeTileA / 2;
            collectionItemR = 15;
        }
    }
    private boolean isSolid(int dx, int dy){
        Layer l = Manager.game.ground.layer;
        Block b = null;
        if(!goingBeyond(dx, dy)) b = (Block) l.getTile((x + dx) / SizeTileA, (y + dy) / SizeTileA);
        if(b != null && b.getType() == Block.SOLID){
            return true;
        }
        return false;
    }
    private boolean goingBeyond(int dx, int dy){
        if(x + dx <= 0){
            return true;
        }
        if(y + dy <= 0){
            return true;
        }
        if(x + dx >= Manager.game.ground.layer.w * SizeTileA){
            return true;
        }
        if(y + dy >= Manager.game.ground.layer.h * SizeTileA){
            return true;
        }
        return false;
    }
}
