package app.world.layer;

import app.Setting;
import app.block.Block;
import java.util.Random;
import ru.wladimir.map.Layer;

/**
 * @author Death
 */
public class Ground {
    
    public static final int 
            WID = 300,
            HEI = 150;

    public static final Object[] FLAT = {WID, HEI, 0, 150 - 100};
    public static final Object[] NORMAL = {WID, HEI, 4, 150 - 100};
    
    public Layer layer;
    
    int stepHeigth;
    int maxHeigth;
    Random rnd;
    int[] pointHeight;
    int[] heigthDirst;
    public long ID_MAP;
    
    public Ground(Object[] o) {
        layer = new Layer((int)o[0], (int)o[1]);
        stepHeigth = (int)o[2];
        maxHeigth = (int)o[3];
        if(Setting.ID_MAP == -1){
            ID_MAP = System.currentTimeMillis();
        }else{
            ID_MAP = Setting.ID_MAP;
        }
        rnd = new Random(ID_MAP);
        pointHeight = new int[layer.w];
        heigthDirst = new int[layer.w];
    }
    
    public void init(){
        for(int i = 0; i < layer.w; i++){
            for(int j = 0; j < layer.h; j++){
                layer.setTile(Block.air, i, j);
            }
        }
        generateMap();
    }
    
    
    public void generateMap(){
        pointHeight[0] = rnd.nextInt(maxHeigth);
        
        for(int i = 1; i < layer.w; i++){
            pointHeight[i] = pointHeight[i - 1] + rnd.nextInt(2 * stepHeigth + 1) - stepHeigth;
            if(pointHeight[i] < maxHeigth)
                pointHeight[i] = maxHeigth;
            else 
                if(pointHeight[i] > layer.h)
                    pointHeight[i] = layer.h - 1;
        }
        
        for(int i = 0; i < layer.w; i++){
            heigthDirst[i] = rnd.nextInt(5) + 5;
        }
        
        int smoothIterations = 5;
        for(int O = 1; O <= smoothIterations; O++){
            for(int i = 1; i < layer.w - 1; i++){
                pointHeight[i] = (pointHeight[i - 1] + pointHeight[i + 1])/2;
                heigthDirst[i] = (heigthDirst[i - 1] + heigthDirst[i + 1])/2;
            }
        }
        
        genrateGrass();
        generateDirst();
        generateTree();
        generateStone();
    }
    
    
    public void genrateGrass(){
        for(int i = 0; i < layer.w; i++){
            layer.setTile(Block.grass.clontIns(), i, pointHeight[i] - 1);
        }
    }
    public void generateDirst(){
        for(int i = 0; i < layer.w; i++){
            layer.setTile(Block.soil.clontIns(), i, pointHeight[i]);
        }
        for(int i = 0; i < layer.w; i++){
            for(int j = pointHeight[i] + 1; j < pointHeight[i] + heigthDirst[i]; j++){
                if(j > 0 && j < layer.h)
                    layer.setTile(Block.dirst.clontIns(), i, j);
            }
        }
    }
    public void generateTree(){
        int heightTree = rnd.nextInt(8) + 4;
        int randomLanding = rnd.nextInt(20);
        final int intend = 6;
        int intendWs = 6;
        for(int i = 0; i < layer.w; i++){
            if(randomLanding == 5){
                    generateTree(i, pointHeight[i], heightTree);
                heightTree = rnd.nextInt(8) + 4;
                randomLanding = rnd.nextInt(20);
                intendWs = i + intend;
            }
            if(i > intendWs)
                randomLanding = rnd.nextInt(20);
        }
    }
    public void generateTree(int x, int y, int h){
        if(x + 5 > layer.w && x - 5 < 0)
            return;
        for(int i = y - 1; i > y - h - 1; i--){
            if(y - 1 - h >= 0)
                layer.setTile(Block.tree.clontIns(), x, i);
        }
        
        layer.setTile(Block.foliage.clontIns(), x , y - h - 1);
        layer.setTile(Block.foliage.clontIns(), x , y - h - 2);
        layer.setTile(Block.foliage.clontIns(), x , y - h - 3);
        
        layer.setTile(Block.foliage.clontIns(), x + 1, y - h);
        layer.setTile(Block.foliage.clontIns(), x + 2, y - h);
        layer.setTile(Block.foliage.clontIns(), x + 3, y - h);
        
        layer.setTile(Block.foliage.clontIns(), x - 1, y - h);
        layer.setTile(Block.foliage.clontIns(), x - 2, y - h);
        layer.setTile(Block.foliage.clontIns(), x - 3, y - h);
        
        layer.setTile(Block.foliage.clontIns(), x + 1, y - h + 1);
        layer.setTile(Block.foliage.clontIns(), x + 2, y - h + 1);
        
        layer.setTile(Block.foliage.clontIns(), x - 1, y - h + 1);
        layer.setTile(Block.foliage.clontIns(), x - 2, y - h + 1);
        
        layer.setTile(Block.foliage.clontIns(), x + 1, y - h + 2);
        
        layer.setTile(Block.foliage.clontIns(), x - 1, y - h + 2);
        
        layer.setTile(Block.foliage.clontIns(), x + 1, y - h - 1);
        layer.setTile(Block.foliage.clontIns(), x + 1, y - h - 2);
        layer.setTile(Block.foliage.clontIns(), x + 2, y - h - 1);
        
        layer.setTile(Block.foliage.clontIns(), x - 1, y - h - 1);
        layer.setTile(Block.foliage.clontIns(), x - 1, y - h - 2);
        layer.setTile(Block.foliage.clontIns(), x - 2, y - h - 1);
    }
    public void generateStone(){
        for(int i = 0; i < layer.w; i++){
            for(int j = pointHeight[i] + heigthDirst[i]; j < layer.h; j++){
                layer.setTile(Block.stone.clontIns(), i, j);
            }
        }
    }
    
}
